# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['modern_treasury',
 'modern_treasury._utils',
 'modern_treasury.resources',
 'modern_treasury.resources.payment_orders',
 'modern_treasury.types',
 'modern_treasury.types.payment_orders']

package_data = \
{'': ['*']}

install_requires = \
['anyio>=3.5.0',
 'distro>=1.7.0',
 'httpx>=0.23.0',
 'pydantic>=1.9.0',
 'typing-extensions>=4.1.1']

setup_kwargs = {
    'name': 'modern-treasury',
    'version': '0.3.0',
    'description': 'Client library for the Modern Treasury API',
    'long_description': '# Modern Treasury Python API Library\n\n[![PyPI version](https://img.shields.io/pypi/v/modern-treasury.svg)](https://pypi.org/project/modern-treasury/)\n\nThe Modern Treasury Python library provides convenient access to the Modern Treasury REST API from any Python 3.7+\napplication. It includes type definitions for all request params and response fields,\nand offers both synchronous and asynchronous clients powered by [httpx](https://github.com/encode/httpx).\n\n![GIF showcasing modern_treasury usage](./showcase.gif)\n\n## Documentation\n\nThe API documentation can be found [here](https://docs.moderntreasury.com).\n\n## Installation\n\n```sh\npip install modern-treasury\n```\n\n## Usage\n\n```python\nfrom modern_treasury import ModernTreasury\n\nmodern_treasury = ModernTreasury(\n    # defaults to os.environ.get("MODERN_TREASURY_API_KEY")\n    api_key="my api key",\n    organization_id="my-organization-ID",\n)\n\nexternal_account = modern_treasury.external_accounts.create({\n    "counterparty_id": "123",\n    "name": "my bank",\n})\nprint(external_account.id)\n```\n\nWhile you can provide an `api_key` keyword argument, we recommend using [python-dotenv](https://pypi.org/project/python-dotenv/)\nand adding `MODERN_TREASURY_API_KEY="my api key"` to your `.env` file so that your API Key is not stored in source control.\n\n## Async Usage\n\nSimply import `AsyncModernTreasury` instead of `ModernTreasury` and use `await` with each API call:\n\n```python\nfrom modern_treasury import AsyncModernTreasury\n\nmodern_treasury = AsyncModernTreasury(\n    # defaults to os.environ.get("MODERN_TREASURY_API_KEY")\n    api_key="my api key",\n    organization_id="my-organization-ID",\n)\n\nasync def main():\n    external_account = await modern_treasury.external_accounts.create({\n        "counterparty_id": "123",\n        "name": "my bank",\n    })\n    print(external_account.id)\n\nasyncio.run(main())\n```\n\nFunctionality between the synchronous and asynchronous clients are otherwise identical.\n\n## Using Types\n\nRequest parameters are [TypedDicts](https://docs.python.org/3/library/typing.html#typing.TypedDict), while responses are [Pydantic](https://pydantic-docs.helpmanual.io/) models. This helps provide autocomplete and documentation within your editor.\n\nIf you would like to see type errors in VS Code to help catch bugs earlier, set `python.analysis.typeCheckingMode` to `"basic"`.\n\n## Pagination\n\nList methods in the Modern Treasury API are paginated.\n\nThis library provides auto-paginating iterators with each list response, so you do not have to request successive pages manually:\n\n```py\nimport modern_treasury\n\nmodern_treasury = ModernTreasury(\n    organization_id="my-organization-ID",\n)\n\nall_external_accounts = []\n# Automatically fetches more pages as needed.\nfor external_account in modern_treasury.external_accounts.list():\n    # Do something with external_account here\n    all_external_accounts.append(external_account)\nreturn all_external_accounts\n```\n\nOr, asynchronously:\n\n```python\nimport asyncio\nimport modern_treasury\n\nmodern_treasury = AsyncModernTreasury(\n    organization_id="my-organization-ID",\n)\n\nasync def main() -> None:\n    all_external_accounts = []\n    # Iterate through items across all pages, issuing requests as needed.\n    async for external_account in modern_treasury.external_accounts.list():\n        all_external_accounts.append(external_account)\n    print(all_external_accounts)\n\nasyncio.run(main())\n```\n\nAlternatively, you can use the `.has_next_page()`, `.next_page_params()`,\nor `.get_next_page()` methods for more granular control working with pages:\n\n```python\nfirst_page = await modern_treasury.external_accounts.list()\nif first_page.has_next_page():\n    print(f"will fetch next page using these details: {first_page.next_page_info()}")\n    next_page = await first_page.get_next_page()\n    print(f"number of items we just fetched: {len(next_page.items)}")\n\n# Remove `await` for non-async usage.\n```\n\nOr just work directly with the returned data:\n\n```python\nfirst_page = await modern_treasury.external_accounts.list()\n\nprint(f"next page cursor: {first_page.after_cursor}") # => "next page cursor: ..."\nfor external_account in first_page.items:\n    print(external_account.id)\n\n# Remove `await` for non-async usage.\n```\n\n## Nested params\n\nNested parameters are dictionaries, typed using `TypedDict`, for example:\n\n```py\nfrom modern_treasury import ModernTreasury\n\nmodern_treasury = ModernTreasury(\n    organization_id="my-organization-ID",\n)\n\nmodern_treasury.external_accounts.create({\n    "foo": {\n        "bar": True\n    },\n})\n```\n\n## Handling errors\n\nWhen the library is unable to connect to the API (e.g., due to network connection problems or a timeout), a subclass of `modern_treasury.APIConnectionError` is raised.\n\nWhen the API returns a non-success status code (i.e., 4xx or 5xx\nresponse), a subclass of `modern_treasury.APIStatusError` will be raised, containing `status_code` and `response` properties.\n\nAll errors inherit from `modern_treasury.APIError`.\n\n```python\nfrom modern_treasury import ModernTreasury\n\nmodern_treasury = ModernTreasury(\n    organization_id="my-organization-ID",\n)\n\ntry:\n    modern_treasury.external_accounts.create({\n        "counterparty_id": "missing",\n    })\nexcept modern_treasury.APIConnectionError as e:\n    print("The server could not be reached")\n    print(e.__cause__) # an underlying Exception, likely raised within httpx.\nexcept modern_treasury.RateLimitError as e:\n    print("A 429 status code was received; we should back off a bit.")\nexcept modern_treasury.APIStatusError as e:\n    print("Another non-200-range status code was received")\n    print(e.status_code)\n    print(e.response)\n```\n\nError codes are as followed:\n\n| Status Code | Error Type                 |\n| ----------- | -------------------------- |\n| 400         | `BadRequestError`          |\n| 401         | `AuthenticationError`      |\n| 403         | `PermissionDeniedError`    |\n| 404         | `NotFoundError`            |\n| 422         | `UnprocessableEntityError` |\n| 429         | `RateLimitError`           |\n| >=500       | `InternalServerError`      |\n| N/A         | `APIConnectionError`       |\n\n### Retries\n\nCertain errors will be automatically retried 2 times by default, with a short exponential backoff.\nConnection errors (for example, due to a network connectivity problem), 409 Conflict, 429 Rate Limit,\nand >=500 Internal errors will all be retried by default.\n\nYou can use the `max_retries` option to configure or disable this:\n\n```python\nfrom modern_treasury import ModernTreasury\n\n# Configure the default for all requests:\nmodern_treasury = ModernTreasury(\n    # default is 2\n    max_retries=0,\n    organization_id="my-organization-ID",\n)\n\n# Or, configure per-request:\nmodern_treasury.external_accounts.list(max_retries=5);\n```\n\n### Timeouts\n\nRequests time out after 60 seconds by default. You can configure this with a `timeout` option,\nwhich accepts a float or an [`httpx.Timeout`](https://www.python-httpx.org/advanced/#fine-tuning-the-configuration):\n\n```python\nfrom modern_treasury import ModernTreasury\n\n# Configure the default for all requests:\nmodern_treasury = ModernTreasury(\n    # default is 60s\n    timeout=20.0,\n    organization_id="my-organization-ID",\n)\n\n# More granular control:\nmodern_treasury = ModernTreasury(\n    timeout=httpx.Timeout(60.0, read=5.0, write=10.0, connect=2.0),\n    organization_id="my-organization-ID",\n)\n\n# Override per-request:\nmodern_treasury.external_accounts.list({\n    "party_name": "my bank",\n}, timeout=5 * 1000)\n```\n\nOn timeout, an `APITimeoutError` is thrown.\n\nNote that requests which time out will be [retried twice by default](#retries).\n\n## Advanced: Configuring custom URLs, proxies, and transports\n\nYou can configure the following keyword arguments when instantiating the client:\n\n```python\nimport httpx\nfrom modern_treasury import ModernTreasury\n\nmodern_treasury = ModernTreasury(\n    # Use a custom base URL\n    base_url="http://my.test.server.example.com:8083",\n    proxies="http://my.test.proxy.example.com",\n    transport=httpx.HTTPTransport(local_address="0.0.0.0"),\n    organization_id="my-organization-ID",\n)\n```\n\nSee the httpx documentation for information about the [`proxies`](https://www.python-httpx.org/advanced/#http-proxying) and [`transport`](https://www.python-httpx.org/advanced/#custom-transports) keyword arguments.\n\n## Status\n\nThis package is in beta. Its internals and interfaces are not stable and subject to change without a major semver bump;\nplease reach out if you rely on any undocumented behavior.\n\nWe are keen for your feedback; please email us at [sdk-feedback@moderntreasury.com](mailto:sdk-feedback@moderntreasury.com) or open an issue with questions,\nbugs, or suggestions.\n\n## Requirements\n\nPython 3.7 or higher.',
    'author': 'Modern Treasury',
    'author_email': 'sdk-feedback@moderntreasury.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Modern-Treasury/modern-treasury-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
